import assert from 'node:assert';
import { parse as parseUrl, format as stringifyUrl } from 'node:url';
import { visit } from 'unist-util-visit';
const remarkRemoveUrlTrailingSlash = function ({
  onlyConsiderHostUrls = false
} = {}, ..._ignored) {
  return tree => {
    visit(tree, ['link', 'image', 'definition'], node => {
      var _parsedUrl$pathname;
      assert(node.type === 'link' || node.type === 'image' || node.type === 'definition', `unexpected node type ${node.type}`);
      const parsedUrl = parseUrl(node.url);
      const target = parsedUrl.protocol && parsedUrl.host ? `${parsedUrl.protocol}//${parsedUrl.host}` : '';
      if (node.url == `${target}/`) {
        node.url = node.url.slice(0, -1);
      } else if (onlyConsiderHostUrls && parsedUrl.pathname == '/' || !onlyConsiderHostUrls && (_parsedUrl$pathname = parsedUrl.pathname) !== null && _parsedUrl$pathname !== void 0 && _parsedUrl$pathname.endsWith('/')) {
        const {
          hash,
          search
        } = parsedUrl;
        parsedUrl.hash = parsedUrl.search = '';
        node.url = `${stringifyUrl(parsedUrl).slice(0, -1)}${search || ''}${hash || ''}`;
      }
    });
  };
};
export default remarkRemoveUrlTrailingSlash;